@@ -5,17 +5,23 @@
 
 #define return_value dst
 
+/* sandbox {{{ */
+#undef TG
+#undef OG
+#define TG(x) (sandbox->tmp_##x)
+#define OG(x) (sandbox->orig_##x)
+/* }}} */
+
 #ifndef HAVE_XCACHE_OPCODE_SPEC_DEF
 #error disassembler cannot be built without xcache/opcode_spec_def.h
 #endif
-static void xc_dasm(zval *dst, zend_op_array *op_array TSRMLS_DC) /* {{{ */
+static void xc_dasm(xc_sandbox_t *sandbox, zval *dst, zend_op_array *op_array TSRMLS_DC) /* {{{ */
 {
 	Bucket *b;
 	zval *zv, *list;
 	xc_compile_result_t cr;
 	int bufsize = 2;
 	char *buf;
-	int keysize;
 
 	xc_compile_result_init_cur(&cr, op_array TSRMLS_CC);
 
@@ -30,15 +36,51 @@ static void xc_dasm(zval *dst, zend_op_array *op_array TSRMLS_DC) /* {{{ */
 	xc_dasm_zend_op_array(zv, op_array TSRMLS_CC);
 	add_assoc_zval_ex(dst, ZEND_STRS("op_array"), zv);
 
+	buf = emalloc(bufsize);
+
 	ALLOC_INIT_ZVAL(list);
 	array_init(list);
-	xc_dasm_HashTable_zend_function(list, CG(function_table) TSRMLS_CC);
+	b = TG(internal_function_tail) ? TG(internal_function_tail)->pListNext : TG(function_table).pListHead;
+	for (; b; b = b->pListNext) {
+		int keysize, keyLength;
+
+		ALLOC_INIT_ZVAL(zv);
+		array_init(zv);
+		xc_dasm_zend_function(zv, b->pData TSRMLS_CC);
+
+		keysize = BUCKET_KEY_SIZE(b) + 2;
+		if (keysize > bufsize) {
+			do {
+				bufsize *= 2;
+			} while (keysize > bufsize);
+			buf = erealloc(buf, bufsize);
+		}
+		memcpy(buf, BUCKET_KEY_S(b), keysize);
+		buf[keysize - 2] = buf[keysize - 1] = ""[0];
+		keyLength = b->nKeyLength;
+#ifdef IS_UNICODE
+		if (BUCKET_KEY_TYPE(b) == IS_UNICODE) {
+			if (buf[0] == ""[0] && buf[1] == ""[0]) {
+				keyLength ++;
+			}
+		} else
+#endif
+		{
+			if (buf[0] == ""[0]) {
+				keyLength ++;
+			}
+		}
+
+		add_u_assoc_zval_ex(list, BUCKET_KEY_TYPE(b), ZSTR(buf), keyLength, zv);
+	}
 	add_assoc_zval_ex(dst, ZEND_STRS("function_table"), list);
 	
-	buf = emalloc(bufsize);
 	ALLOC_INIT_ZVAL(list);
 	array_init(list);
-	for (b = CG(class_table)->pListHead; b; b = b->pListNext) {
+	b = TG(internal_class_tail) ? TG(internal_class_tail)->pListNext : TG(class_table).pListHead;
+	for (; b; b = b->pListNext) {
+		int keysize, keyLength;
+
 		ALLOC_INIT_ZVAL(zv);
 		array_init(zv);
 		xc_dasm_zend_class_entry(zv, CestToCePtr(*(xc_cest_t *)b->pData) TSRMLS_CC);
@@ -52,20 +94,20 @@ static void xc_dasm(zval *dst, zend_op_array *op_array TSRMLS_DC) /* {{{ */
 		}
 		memcpy(buf, BUCKET_KEY_S(b), keysize);
 		buf[keysize - 2] = buf[keysize - 1] = ""[0];
-		keysize = b->nKeyLength;
+		keyLength = b->nKeyLength;
 #ifdef IS_UNICODE
 		if (BUCKET_KEY_TYPE(b) == IS_UNICODE) {
 			if (buf[0] == ""[0] && buf[1] == ""[0]) {
-				keysize ++;
+				keyLength ++;
 			}
 		} else
 #endif
 		{
 			if (buf[0] == ""[0]) {
-				keysize ++;
+				keyLength ++;
 			}
 		}
-		add_u_assoc_zval_ex(list, BUCKET_KEY_TYPE(b), ZSTR(buf), b->nKeyLength, zv);
+		add_u_assoc_zval_ex(list, BUCKET_KEY_TYPE(b), ZSTR(buf), keyLength, zv);
 	}
 	efree(buf);
 	add_assoc_zval_ex(dst, ZEND_STRS("class_table"), list);
@@ -96,7 +138,7 @@ void xc_dasm_string(zval *dst, zval *source TSRMLS_DC) /* {{{ */
 		goto err_compile;
 	}
 
-	xc_dasm(dst, op_array TSRMLS_CC);
+	xc_dasm(&sandbox, dst, op_array TSRMLS_CC);
 
 	/* free */
 	efree(eval_name);
@@ -141,7 +183,7 @@ void xc_dasm_file(zval *dst, const char *filename TSRMLS_DC) /* {{{ */
 		goto err_compile;
 	}
 
-	xc_dasm(dst, op_array TSRMLS_CC);
+	xc_dasm(&sandbox, dst, op_array TSRMLS_CC);
 
 	/* free */
 #ifdef ZEND_ENGINE_2
