@@ -930,8 +930,8 @@ static void xc_cache_early_binding_class_cb(zend_op *opline, int oplineno, void
 	xc_cest_t cest;
 	xc_entry_data_php_t *php = (xc_entry_data_php_t *) data;
 
-	class_name = opline->op1.u.constant.value.str.val;
-	class_len  = opline->op1.u.constant.value.str.len;
+	class_name = Z_OP_CONSTANT(opline->op1).value.str.val;
+	class_len  = Z_OP_CONSTANT(opline->op1).value.str.len;
 	if (zend_hash_find(CG(class_table), class_name, class_len, (void **) &cest) == FAILURE) {
 		assert(0);
 	}
@@ -1000,7 +1000,7 @@ static zend_op_array *xc_compile_file(zend_file_handle *h, int type TSRMLS_DC) /
 	/* }}} */
 	/* {{{ restore */
 	/* stale precheck */
-	if (cache->compiling) {
+	if (XG(request_time) - cache->compiling < 30) {
 		cache->clogs ++; /* is it safe here? */
 		return old_compile_file(h, type TSRMLS_CC);
 	}
@@ -1009,7 +1009,7 @@ static zend_op_array *xc_compile_file(zend_file_handle *h, int type TSRMLS_DC) /
 	op_array = NULL;
 	ENTER_LOCK_EX(cache) {
 		/* clogged */
-		if (cache->compiling) {
+		if (XG(request_time) - cache->compiling < 30) {
 			cache->clogs ++;
 			op_array = NULL;
 			clogged = 1;
@@ -2279,6 +2279,34 @@ PHP_FUNCTION(xcache_dec)
 	xc_var_inc_dec(-1, INTERNAL_FUNCTION_PARAM_PASSTHRU);
 }
 /* }}} */
+/* {{{ proto int xcache_get_refcount(mixed variable)
+   XCache internal uses only: Get reference count of variable */
+PHP_FUNCTION(xcache_get_refcount)
+{
+	zval *variable;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &variable) == FAILURE) {
+		RETURN_NULL();
+	}
+
+	RETURN_LONG(Z_REFCOUNT(*variable));
+}
+/* }}} */
+/* {{{ proto bool xcache_get_isref(mixed variable)
+   XCache internal uses only: Check if variable data is marked referenced */
+ZEND_BEGIN_ARG_INFO_EX(arginfo_xcache_get_isref, 0, 0, 1)
+	ZEND_ARG_INFO(1, variable)
+ZEND_END_ARG_INFO()
+
+PHP_FUNCTION(xcache_get_isref)
+{
+	zval *variable;
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &variable) == FAILURE) {
+		RETURN_NULL();
+	}
+
+	RETURN_BOOL(Z_ISREF(*variable) && Z_REFCOUNT(*variable) >= 3);
+}
+/* }}} */
 #ifdef HAVE_XCACHE_DPRINT
 /* {{{ proto bool  xcache_dprint(mixed value)
    Prints variable (or value) internal struct (debug only) */
@@ -2427,7 +2455,8 @@ PHP_FUNCTION(xcache_get_opcode_spec)
 }
 /* }}} */
 #endif
-/* {{{ proto mixed xcache_get_special_value(zval value) */
+/* {{{ proto mixed xcache_get_special_value(zval value)
+   XCache internal use only: For decompiler to get static value with type fixed */
 PHP_FUNCTION(xcache_get_special_value)
 {
 	zval *value;
@@ -2436,21 +2465,35 @@ PHP_FUNCTION(xcache_get_special_value)
 		return;
 	}
 
-	if (value->type == IS_CONSTANT) {
+	switch ((Z_TYPE_P(value) & IS_CONSTANT_TYPE_MASK)) {
+	case IS_CONSTANT:
 		*return_value = *value;
 		zval_copy_ctor(return_value);
 		return_value->type = UNISW(IS_STRING, UG(unicode) ? IS_UNICODE : IS_STRING);
-		return;
-	}
+		break;
 
-	if (value->type == IS_CONSTANT_ARRAY) {
+	case IS_CONSTANT_ARRAY:
 		*return_value = *value;
 		zval_copy_ctor(return_value);
 		return_value->type = IS_ARRAY;
+		break;
+
+	default:
+		RETURN_NULL();
+	}
+}
+/* }}} */
+/* {{{ proto int xcache_get_type(zval value)
+   XCache internal use only for disassembler to get variable type in engine level */
+PHP_FUNCTION(xcache_get_type)
+{
+	zval *value;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "z", &value) == FAILURE) {
 		return;
 	}
 
-	RETURN_NULL();
+	RETURN_LONG(Z_TYPE_P(value));
 }
 /* }}} */
 /* {{{ proto string xcache_coredump(int op_type) */
@@ -2477,7 +2520,7 @@ PHP_FUNCTION(xcache_is_autoglobal)
 	RETURN_BOOL(zend_hash_exists(CG(auto_globals), name, name_len + 1));
 }
 /* }}} */
-static function_entry xcache_functions[] = /* {{{ */
+static zend_function_entry xcache_functions[] = /* {{{ */
 {
 	PHP_FE(xcache_count,             NULL)
 	PHP_FE(xcache_info,              NULL)
@@ -2505,6 +2548,7 @@ static function_entry xcache_functions[] = /* {{{ */
 	PHP_FE(xcache_coverager_get,     NULL)
 #endif
 	PHP_FE(xcache_get_special_value, NULL)
+	PHP_FE(xcache_get_type,          NULL)
 	PHP_FE(xcache_get_op_type,       NULL)
 	PHP_FE(xcache_get_data_type,     NULL)
 	PHP_FE(xcache_get_opcode,        NULL)
@@ -2519,6 +2563,8 @@ static function_entry xcache_functions[] = /* {{{ */
 	PHP_FE(xcache_isset,             NULL)
 	PHP_FE(xcache_unset,             NULL)
 	PHP_FE(xcache_unset_by_prefix,   NULL)
+	PHP_FE(xcache_get_refcount,      NULL)
+	PHP_FE(xcache_get_isref,         arginfo_xcache_get_isref)
 #ifdef HAVE_XCACHE_DPRINT
 	PHP_FE(xcache_dprint,            NULL)
 #endif
@@ -2965,10 +3011,12 @@ static PHP_MSHUTDOWN_FUNCTION(xcache)
 
 	if (xc_zend_extension_faked) {
 		zend_extension *ext = zend_get_extension(XCACHE_NAME);
-		if (ext->shutdown) {
-			ext->shutdown(ext);
+		if (ext) {
+			if (ext->shutdown) {
+				ext->shutdown(ext);
+			}
+			xc_zend_remove_extension(ext);
 		}
-		xc_zend_remove_extension(ext);
 	}
 	UNREGISTER_INI_ENTRIES();
 
