@@ -17,6 +17,13 @@ ZEND_BEGIN_MODULE_GLOBALS(xcache)
 	long   var_ttl;
 	zend_bool auth_enabled;
 
+#ifdef ZEND_ENGINE_2
+	HashTable gc_op_arrays;
+#endif
+
+#ifdef HAVE_XCACHE_CONSTANT
+	HashTable internal_constant_table;
+#endif
 	HashTable internal_function_table;
 	HashTable internal_class_table;
 	zend_bool internal_table_copied;
